{******************************************************************************}
{*                                                                            *}
{*  Copyright (c) 2002, NVIDIA Corporation.                                   *}
{*                                                                            *}
{*  Files:    cg.h, cg_datatypes.h, cg_errors.h, cg_profiles.h,               *}
{*            cgGL_profiles.h, cg_bindlocations.h                             *}
{*  Content:  NVIDIA Cg core include files                                    *}
{*                                                                            *}
{*  NVIDIA "Cg" ObjectPascal adaptation by Alexey Barkovoy                    *}
{*  E-Mail: clootie@reactor.ru                                                *}
{*                                                                            *}
{*  Modified: 26-Aug-2002                                                     *}
{*                                                                            *}
{*  Latest version can be downloaded from:                                    *}
{*     http://clootie.narod.ru                                                *}
{*     http://delphi-jedi.org                                                 *}
{*                                                                            *}
{******************************************************************************}
{* Known Bugs:                                                                *}
{* cgDuplicateProgram - is not found in any lib's provided by NVIDIA          *}
{******************************************************************************}
{*                                                                            *}
{******************************************************************************}
{                                                                              }
{ Obtained through: Joint Endeavour of Delphi Innovators (Project JEDI)        }
{                                                                              }
{ The contents of this file are used with permission, subject to the Mozilla   }
{ Public License Version 1.1 (the "License"); you may not use this file except }
{ in compliance with the License. You may obtain a copy of the License at      }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ Alternatively, the contents of this file may be used under the terms of the  }
{ GNU Lesser General Public License (the  "LGPL License"), in which case the   }
{ provisions of the LGPL License are applicable instead of those above.        }
{ If you wish to allow use of your version of this file only under the terms   }
{ of the LGPL License and not to allow others to use your version of this file }
{ under the MPL, indicate your decision by deleting  the provisions above and  }
{ replace  them with the notice and other provisions required by the LGPL      }
{ License.  If you do not delete the provisions above, a recipient may use     }
{ your version of this file under either the MPL or the LGPL License.          }
{                                                                              }
{ For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html }
{                                                                              }
{******************************************************************************}

(*
 *
 * Copyright (c) 2002, NVIDIA Corporation.
 *
 *
 *
 * NVIDIA Corporation("NVIDIA") supplies this software to you in consideration
 * of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this NVIDIA software constitutes
 * acceptance of these terms.  If you do not agree with these terms, please do
 * not use, install, modify or redistribute this NVIDIA software.
 *
 *
 *
 * In consideration of your agreement to abide by the following terms, and
 * subject to these terms, NVIDIA grants you a personal, non-exclusive license,
 * under NVIDIAs copyrights in this original NVIDIA software (the "NVIDIA
 * Software"), to use, reproduce, modify and redistribute the NVIDIA
 * Software, with or without modifications, in source and/or binary forms;
 * provided that if you redistribute the NVIDIA Software, you must retain the
 * copyright notice of NVIDIA, this notice and the following text and
 * disclaimers in all such redistributions of the NVIDIA Software. Neither the
 * name, trademarks, service marks nor logos of NVIDIA Corporation may be used
 * to endorse or promote products derived from the NVIDIA Software without
 * specific prior written permission from NVIDIA.  Except as expressly stated
 * in this notice, no other rights or licenses express or implied, are granted
 * by NVIDIA herein, including but not limited to any patent rights that may be
 * infringed by your derivative works or by other works in which the NVIDIA
 * Software may be incorporated. No hardware is licensed hereunder.
 *
 *
 *
 * THE NVIDIA SOFTWARE IS BEING PROVIDED ON AN "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
 * WITHOUT LIMITATION, WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR ITS USE AND OPERATION
 * EITHER ALONE OR IN COMBINATION WITH OTHER PRODUCTS.
 *
 *
 *
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * EXEMPLARY, CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOST
 * PROFITS; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) OR ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE NVIDIA SOFTWARE,
 * HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING
 * NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF NVIDIA HAS BEEN ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

//////////////////////////////////////////////////////////////////////////////
// HISTORY:
// 26-Aug-02 - Alexey Barkovoy:
//   - Updated to Beta2 of Cg toolkit (compiler version 1.5)
// 01-Jul-02 - Alexey Barkovoy:
//   - Changed to INC file
// 18-Jun-02 - Alexey Barkovoy:
//   - TcgProfileType corrected

{$Include JEDI.inc}
{$MINENUMSIZE 4}

(*$HPPEMIT '#include "cg.h"' *)

const
{$IFDEF cg_DirectX}
  CgLibrary = 'cgD3D_ab.dll';
{$ELSE} // cg_OpenGL
  CgLibrary = 'cgGL_ab.dll';
{$ENDIF}

(*************************************************************************)
(*** CG Run-Time Library API                                          ***)
(*************************************************************************)

(*************************************************************************)
(*** Data types and structures                                         ***)
(*************************************************************************)

type
  (*
   * The following define error codes returned by various cg
   * API functions.
   *
   *   CG_ERROR_MACRO(code, enum_name, message)
   *
   *     code      : The integer error code associated with the error.
   *     enum_name : The name of enumerant of the error code.
   *                  (Note : the prefix "cg" will be added to the name).
   *     message   : A description string associated with the error.
   *
   *)
  TcgError = (
    //# define CG_ERROR_MACRO(code, enum_name, message) \
    //   cg##enum_name = code,
    //# include <Cg/cg_errors.h>
    cgNoError                   {$IFDEF COMPILER6_UP}= 0{$ENDIF},  // "No error has occurred."
    cgCompileError              {$IFDEF COMPILER6_UP}= 1{$ENDIF},  // "The compile returned an error."
    cgInvalidParameterError     {$IFDEF COMPILER6_UP}= 2{$ENDIF},  // "The parameter used is of invalid type."
    cgInvalidProfileError       {$IFDEF COMPILER6_UP}= 3{$ENDIF},  // "The profile is not supported."
    cgProgramLoadError          {$IFDEF COMPILER6_UP}= 4{$ENDIF},  // "The program did could not load."
    cgProgramBindError          {$IFDEF COMPILER6_UP}= 5{$ENDIF},  // "The program could not bind."
    cgProgramNotLoadedError     {$IFDEF COMPILER6_UP}= 6{$ENDIF},  // "The program must be loaded before this operation may be used."
    cgUnsupportedGLExtensionError{$IFDEF COMPILER6_UP}=7{$ENDIF},  // "An unsupported GL extension was required to perform this operation."
    cgInvalidValueTypeError     {$IFDEF COMPILER6_UP}= 8{$ENDIF},  // "An unknown value type was assigned to a parameter."
    cgNotMatrixParamError       {$IFDEF COMPILER6_UP}= 9{$ENDIF},  // "The parameter is not of matrix type."
    cgInvalidEnumerantError     {$IFDEF COMPILER6_UP}= 10{$ENDIF}, // "The enumerant parameter has an invalid value."
    cgNot4x4MatrixError         {$IFDEF COMPILER6_UP}= 11{$ENDIF}, // "The parameter must be a 4x4 matrix type."
    cgFileReadError             {$IFDEF COMPILER6_UP}= 12{$ENDIF}, // "The could not be read."
    cgFileWriteError            {$IFDEF COMPILER6_UP}= 13{$ENDIF}  // "The could not be written."
  );
  cgError = TcgError;
  {$NODEFINE cgError}
  {$NODEFINE TcgError}
  {$HPPEMIT 'typedef cgError TcgError;'}

  PcgContext = ^TcgContext;
  TcgContext = packed record
    (* Opaque structure *)
    Data: Pointer;
  end;
  cgContext = TcgContext;
  {$NODEFINE cgContext}
  {$NODEFINE TcgContext}
  {$HPPEMIT 'typedef cgContext TcgContext;'}

  PcgProgramIter = ^TcgProgramIter;
  TcgProgramIter = packed record
    (* Opaque structure *)
    Data: Pointer;
  end;
  cgProgramIter = TcgProgramIter;
  {$NODEFINE cgProgramIter}
  {$NODEFINE TcgProgramIter}
  {$HPPEMIT 'typedef cgProgramIter TcgProgramIter;'}

  PcgBindIter = ^TcgBindIter;
  TcgBindIter = packed record
    (* Opaque structure *)
    Data: Pointer;
  end;
  cgBindIter = TcgBindIter;
  {$NODEFINE cgBindIter}
  {$NODEFINE TcgBindIter}
  {$HPPEMIT 'typedef cgBindIter TcgBindIter;'}


  (*
   * The following define the supported CG basic data types.
   *
   *   CG_DATATYPE_MACRO(name, compiler_name, nrows, ncols)
   *
   *     name          : The name of the data type.
   *     compiler_name : The name of the data type within the compiler syntax.
   *     nrows         : Number of rows for matrix types.  Should be 0 other-wise.
   *     ncols         : Number of columns for matrix types.  Should be 0
   *                     other-wise.
   *
   *)
  TcgValueType = (
    // # define CG_DATATYPE_MACRO(name, compiler_name, ncols, nrows) \
    //    cg##name##ValueType,
    cgHalfValueType,      // CG_DATATYPE_MACRO(Half,half,0,0)
    cgHalf2ValueType,     // CG_DATATYPE_MACRO(Half2,half2,0,0)
    cgHalf3ValueType,     // CG_DATATYPE_MACRO(Half3,half3,0,0)
    cgHalf4ValueType,     // CG_DATATYPE_MACRO(Half4,half4,0,0)
    cgHalf1x1ValueType,   // CG_DATATYPE_MACRO(Half1x1,half1x1,1,1)
    cgHalf1x2ValueType,   // CG_DATATYPE_MACRO(Half1x2,half1x2,1,2)
    cgHalf1x3ValueType,   // CG_DATATYPE_MACRO(Half1x3,half1x3,1,3)
    cgHalf1x4ValueType,   // CG_DATATYPE_MACRO(Half1x4,half1x4,1,4)
    cgHalf2x1ValueType,   // CG_DATATYPE_MACRO(Half2x1,half2x1,2,1)
    cgHalf2x2ValueType,   // CG_DATATYPE_MACRO(Half2x2,half2x2,2,2)
    cgHalf2x3ValueType,   // CG_DATATYPE_MACRO(Half2x3,half2x3,2,3)
    cgHalf2x4ValueType,   // CG_DATATYPE_MACRO(Half2x4,half2x4,2,4)
    cgHalf3x1ValueType,   // CG_DATATYPE_MACRO(Half3x1,half3x1,3,1)
    cgHalf3x2ValueType,   // CG_DATATYPE_MACRO(Half3x2,half3x2,3,2)
    cgHalf3x3ValueType,   // CG_DATATYPE_MACRO(Half3x3,half3x3,3,3)
    cgHalf3x4ValueType,   // CG_DATATYPE_MACRO(Half3x4,half3x4,3,4)
    cgHalf4x1ValueType,   // CG_DATATYPE_MACRO(Half4x1,half4x1,4,1)
    cgHalf4x2ValueType,   // CG_DATATYPE_MACRO(Half4x2,half4x2,4,2)
    cgHalf4x3ValueType,   // CG_DATATYPE_MACRO(Half4x3,half4x3,4,3)
    cgHalf4x4ValueType,   // CG_DATATYPE_MACRO(Half4x4,half4x4,4,4)
    cgFloatValueType,     // CG_DATATYPE_MACRO(Float,float,0,0)
    cgFloat2ValueType,    // CG_DATATYPE_MACRO(Float2,float2,0,0)
    cgFloat3ValueType,    // CG_DATATYPE_MACRO(Float3,float3,0,0)
    cgFloat4ValueType,    // CG_DATATYPE_MACRO(Float4,float4,0,0)
    cgFloat1x1ValueType,  // CG_DATATYPE_MACRO(Float1x1,float1x1,1,1)
    cgFloat1x2ValueType,  // CG_DATATYPE_MACRO(Float1x2,float1x2,1,2)
    cgFloat1x3ValueType,  // CG_DATATYPE_MACRO(Float1x3,float1x3,1,3)
    cgFloat1x4ValueType,  // CG_DATATYPE_MACRO(Float1x4,float1x4,1,4)
    cgFloat2x1ValueType,  // CG_DATATYPE_MACRO(Float2x1,float2x1,2,1)
    cgFloat2x2ValueType,  // CG_DATATYPE_MACRO(Float2x2,float2x2,2,2)
    cgFloat2x3ValueType,  // CG_DATATYPE_MACRO(Float2x3,float2x3,2,3)
    cgFloat2x4ValueType,  // CG_DATATYPE_MACRO(Float2x4,float2x4,2,4)
    cgFloat3x1ValueType,  // CG_DATATYPE_MACRO(Float3x1,float3x1,3,1)
    cgFloat3x2ValueType,  // CG_DATATYPE_MACRO(Float3x2,float3x2,3,2)
    cgFloat3x3ValueType,  // CG_DATATYPE_MACRO(Float3x3,float3x3,3,3)
    cgFloat3x4ValueType,  // CG_DATATYPE_MACRO(Float3x4,float3x4,3,4)
    cgFloat4x1ValueType,  // CG_DATATYPE_MACRO(Float4x1,float4x1,4,1)
    cgFloat4x2ValueType,  // CG_DATATYPE_MACRO(Float4x2,float4x2,4,2)
    cgFloat4x3ValueType,  // CG_DATATYPE_MACRO(Float4x3,float4x3,4,3)
    cgFloat4x4ValueType,  // CG_DATATYPE_MACRO(Float4x4,float4x4,4,4)
    cgSampler1DValueType, // CG_DATATYPE_MACRO(Sampler1D,sampler1D,0,0)
    cgSampler2DValueType, // CG_DATATYPE_MACRO(Sampler2D,sampler2D,0,0)
    cgSampler3DValueType, // CG_DATATYPE_MACRO(Sampler3D,sampler3D,0,0)
    cgSamplerRECTValueType, // CG_DATATYPE_MACRO(SamplerRECT,samplerRECT,0,0)
    cgSamplerCUBEValueType, // CG_DATATYPE_MACRO(SamplerCUBE,samplerCUBE,0,0)

    cgUnknownValueType
  );
  cgValueType = TcgValueType;
  {$NODEFINE cgValueType}
  {$NODEFINE TcgValueType}
  {$HPPEMIT 'typedef cgValueType TcgValueType;'}

  TcgParamType = (
    cgConstParam,
    cgUniformParam,
    cgConnectorMemberParam,
{$IFDEF COMPILER6_UP}
    cgVaryingParam = cgConnectorMemberParam,
{$ENDIF}
    cgTexObjParam,
    cgAnyParam,
    cgUndefinedParam
  );
  cgParamType = TcgParamType;
  {$NODEFINE cgParamType}
  {$NODEFINE TcgParamType}
  {$HPPEMIT 'typedef cgParamType TcgParamType;'}

{$IFNDEF COMPILER6_UP}
const
  cgVaryingParam = cgConnectorMemberParam;

type
{$ENDIF}
(* 
 * The following macro invocations define the supported CG basic hardware
 * bind locations.
 *
 * The macros have the form :
 *
 *   CG_BINDLOCATION_MACRO(name, compiler_name, enum_int)
 *
 *     name          : The name of the location.
 *     compiler_name : The name of the location within the compiler syntax.
 *     int_id        : Integer enumerant associated with this bind location.
 *     addressable   : The bind location must have an integer address
 *                     associated with it.
 *     ParamType     : the cgParamType of this register.
 *
 *)
  TcgBindLocation = (
    //# define CG_BINDLOCATION_MACRO(name,compiler_name,\
    //                               enum_int,addressable,param_type) \
    //  cgBind##name = enum_int,
    cgBindTexUnit0{$IFDEF COMPILER6_UP} = 0{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit0,"texunit 0",0,0,cgTexObjParam)
    cgBindTexUnit1{$IFDEF COMPILER6_UP} = 1{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit1,"texunit 1",1,0,cgTexObjParam)
    cgBindTexUnit2{$IFDEF COMPILER6_UP} = 2{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit2,"texunit 2",2,0,cgTexObjParam)
    cgBindTexUnit3{$IFDEF COMPILER6_UP} = 3{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit3,"texunit 3",3,0,cgTexObjParam)
    cgBindTexUnit4{$IFDEF COMPILER6_UP} = 4{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit4,"texunit 4",4,0,cgTexObjParam)
    cgBindTexUnit5{$IFDEF COMPILER6_UP} = 5{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit5,"texunit 5",5,0,cgTexObjParam)
    cgBindTexUnit6{$IFDEF COMPILER6_UP} = 6{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit6,"texunit 6",6,0,cgTexObjParam)
    cgBindTexUnit7{$IFDEF COMPILER6_UP} = 7{$ENDIF}, // CG_BINDLOCATION_MACRO(TexUnit7,"texunit 7",7,0,cgTexObjParam)

    cgBindAttr0{$IFDEF COMPILER6_UP} = 8{$ENDIF},    // CG_BINDLOCATION_MACRO(Attr0,"ATTR0",8,0,cgConnectorMemberParam)
    cgBindAttr1{$IFDEF COMPILER6_UP} = 9{$ENDIF},    // CG_BINDLOCATION_MACRO(Attr1,"ATTR1",9,0,cgConnectorMemberParam)
    cgBindAttr2{$IFDEF COMPILER6_UP} = 10{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr2,"ATTR2",10,0,cgConnectorMemberParam)
    cgBindAttr3{$IFDEF COMPILER6_UP} = 11{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr3,"ATTR3",11,0,cgConnectorMemberParam)
    cgBindAttr4{$IFDEF COMPILER6_UP} = 12{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr4,"ATTR4",12,0,cgConnectorMemberParam)
    cgBindAttr5{$IFDEF COMPILER6_UP} = 13{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr5,"ATTR5",13,0,cgConnectorMemberParam)
    cgBindAttr6{$IFDEF COMPILER6_UP} = 14{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr6,"ATTR6",14,0,cgConnectorMemberParam)
    cgBindAttr7{$IFDEF COMPILER6_UP} = 15{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr7,"ATTR7",15,0,cgConnectorMemberParam)
    cgBindAttr8{$IFDEF COMPILER6_UP} = 16{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr8,"ATTR8",16,0,cgConnectorMemberParam)
    cgBindAttr9{$IFDEF COMPILER6_UP} = 17{$ENDIF},   // CG_BINDLOCATION_MACRO(Attr9,"ATTR9",17,0,cgConnectorMemberParam)
    cgBindAttr10{$IFDEF COMPILER6_UP} = 18{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr10,"ATTR10",18,0,cgConnectorMemberParam)
    cgBindAttr11{$IFDEF COMPILER6_UP} = 19{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr11,"ATTR11",19,0,cgConnectorMemberParam)
    cgBindAttr12{$IFDEF COMPILER6_UP} = 20{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr12,"ATTR12",20,0,cgConnectorMemberParam)
    cgBindAttr13{$IFDEF COMPILER6_UP} = 21{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr13,"ATTR13",21,0,cgConnectorMemberParam)
    cgBindAttr14{$IFDEF COMPILER6_UP} = 22{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr14,"ATTR14",22,0,cgConnectorMemberParam)
    cgBindAttr15{$IFDEF COMPILER6_UP} = 23{$ENDIF},  // CG_BINDLOCATION_MACRO(Attr15,"ATTR15",23,0,cgConnectorMemberParam)

    cgBindVertUniform{$IFDEF COMPILER6_UP} = 24{$ENDIF}, // CG_BINDLOCATION_MACRO(VertUniform,"c",24,1,cgUniformParam)

    cgBindTex0{$IFDEF COMPILER6_UP} = 25{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex0,"TEX0",25,0,cgConnectorMemberParam)
    cgBindTex1{$IFDEF COMPILER6_UP} = 26{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex1,"TEX1",26,0,cgConnectorMemberParam)
    cgBindTex2{$IFDEF COMPILER6_UP} = 27{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex2,"TEX2",27,0,cgConnectorMemberParam)
    cgBindTex3{$IFDEF COMPILER6_UP} = 28{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex3,"TEX3",28,0,cgConnectorMemberParam)
    cgBindTex4{$IFDEF COMPILER6_UP} = 29{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex4,"TEX4",29,0,cgConnectorMemberParam)
    cgBindTex5{$IFDEF COMPILER6_UP} = 30{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex5,"TEX5",30,0,cgConnectorMemberParam)
    cgBindTex6{$IFDEF COMPILER6_UP} = 31{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex6,"TEX6",31,0,cgConnectorMemberParam)
    cgBindTex7{$IFDEF COMPILER6_UP} = 32{$ENDIF},    // CG_BINDLOCATION_MACRO(Tex7,"TEX7",32,0,cgConnectorMemberParam)

    cgBindHPos{$IFDEF COMPILER6_UP} = 33{$ENDIF},    // CG_BINDLOCATION_MACRO(HPos,"HPOS",33,0,cgConnectorMemberParam)
    cgBindCol0{$IFDEF COMPILER6_UP} = 34{$ENDIF},    // CG_BINDLOCATION_MACRO(Col0,"COL0",34,0,cgConnectorMemberParam)
    cgBindCol1{$IFDEF COMPILER6_UP} = 35{$ENDIF},    // CG_BINDLOCATION_MACRO(Col1,"COL1",35,0,cgConnectorMemberParam)
    cgBindCol{$IFDEF COMPILER6_UP} = 36{$ENDIF},     // CG_BINDLOCATION_MACRO(Col,"COL",36,0,cgConnectorMemberParam)
    cgBindCol2{$IFDEF COMPILER6_UP} = 37{$ENDIF},    // CG_BINDLOCATION_MACRO(Col2,"COL2",37,0,cgConnectorMemberParam)
    cgBindCol3{$IFDEF COMPILER6_UP} = 38{$ENDIF},    // CG_BINDLOCATION_MACRO(Col3,"COL3",38,0,cgConnectorMemberParam)
    cgBindPSiz{$IFDEF COMPILER6_UP} = 39{$ENDIF},    // CG_BINDLOCATION_MACRO(PSiz,"PSIZ",39,0,cgConnectorMemberParam)
    cgBindWPos{$IFDEF COMPILER6_UP} = 40{$ENDIF},    // CG_BINDLOCATION_MACRO(WPos,"WPOS",40,0,cgConnectorMemberParam)

    cgBindUndefined
  );
  cgBindLocation = TcgBindLocation;
  {$NODEFINE cgBindLocation}
  {$NODEFINE TcgBindLocation}
  {$HPPEMIT 'typedef cgBindLocation TcgBindLocation;'}


  (*
   * The following define the supported CG profiles.
   *
   *   CG_PROFILE_MACRO(name, compiler_id, compiler_opt)
   *
   *     name         : The name of the profile.  Used consistently with the API.
   *     compiler_id  : The identifier string for the profile used by the compiler.
   *     compiler_opt : The command-line switch used to force compilation into
   *                    the profile.
   *
   *)
  TcgProfileType = (
    // # define CG_PROFILE_MACRO(name, compiler_id, compiler_opt) \
    //    cg##name##Profile,
    // # include "Cg/cg_profiles.h"
    cgVertexProfile,            // CG_PROFILE_MACRO(Vertex,vp20,"vp20")
    cgFragment20Profile,        // CG_PROFILE_MACRO(Fragment20,fp20,"dx8ps")
    cgVertex30Profile,          // CG_PROFILE_MACRO(Vertex30,vp30,"vp30")
    cgFragmentProfile,          // CG_PROFILE_MACRO(Fragment,fp30,"fp30")
    cgARBVertexProfile,         // CG_PROFILE_MACRO(ARBVertex,arbvp1,"arbvp1")
    cgDX8VertexProfile,         // CG_PROFILE_MACRO(DX8Vertex,dx8vs,"dx8vs")
    cgDX8PixelProfile,          // CG_PROFILE_MACRO(DX8Pixel,dx8ps,"dx8ps")

    cgNProfiles,
    cgUnknownProfile
  );
  cgProfileType = TcgProfileType;
  {$NODEFINE cgProfileType}
  {$NODEFINE TcgProfileType}
  {$HPPEMIT 'typedef cgProfileType TcgProfileType;'}


(*************************************************************************)
(*** Function Prototypes                                               ***)
(*************************************************************************)


(*******************************************************************)
(*** cg Misc Functions                                         ****)
(*******************************************************************)

procedure cgCleanup; cdecl; external CgLibrary;
{$EXTERNALSYM cgCleanup}

(* This will probably be deprecated in future releases *)
procedure cgSetCompilerExe(const exe: PChar); cdecl; external CgLibrary;
{$EXTERNALSYM cgSetCompilerExe}

// extern const char *cgErrorMsg(cgError error);
function cgErrorMsg(error: TcgError): PChar { const }; cdecl; external CgLibrary;
{$EXTERNALSYM cgErrorMsg}

(*******************************************************************)
(*** cgContext Functions                                       ****)
(*******************************************************************)

function cgCreateContext: PcgContext; cdecl; external CgLibrary;
{$EXTERNALSYM cgCreateContext}

procedure cgFreeContext(context: PcgContext); cdecl; external CgLibrary;
{$EXTERNALSYM cgFreeContext}

function cgAddProgram(context: PcgContext; const text: PChar;
  profile: TcgProfileType; const entry: PChar): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgAddProgram}

function cgAddProgramFromFile(context: PcgContext; const filename: PChar;
  profile: TcgProfileType; const entry: PChar): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgAddProgramFromFile}

type
  PPCharArray = ^TPCharArray;
  TPCharArray = array[0..MaxInt div SizeOf(PChar) - 1] of PChar;
  PPChar = ^PChar; //Clootie: It's actually pointer to array of PChar strings

function cgAddProgramArgs(context: PcgContext; const text: PChar;
  profile: TcgProfileType; const entry: PChar;
  nargs: Integer; const args: PPChar): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgAddProgramArgs}

function cgAddProgramFromFileArgs(context: PcgContext; const filename: PChar;
  profile: TcgProfileType; const entry: PChar;
  nargs: Integer; const args: PPChar): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgAddProgramFromFileArgs}

function cgGetLastListing(context: PcgContext): PChar { const }; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetLastListing}

(*******************************************************************)
(*** cgProgram Functions                                        ***)
(*******************************************************************)

function cgGetNextProgram(context: PcgContext; iter: PcgProgramIter): PcgProgramIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetNextProgram}

function cgProgramByName(context: PcgContext; const name: PChar): PcgProgramIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgProgramByName}

function cgGetLastProgram(context: PcgContext): PcgProgramIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetLastProgram}

procedure cgFreeProgramIter(program_: PcgProgramIter); cdecl; external CgLibrary;
{$EXTERNALSYM cgFreeProgramIter}

function cgDuplicateProgram(program_: PcgProgramIter): PcgProgramIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgDuplicateProgram}

function cgGetProgramName(const program_: PcgProgramIter): PChar { const }; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetProgramName}

function cgGetProgramProfile(const program_: PcgProgramIter): TcgProfileType; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetProgramProfile}

function cgGetProgramObjectCode(const program_: PcgProgramIter): PChar { const }; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetProgramObjectCode}



(*******************************************************************)
(*** Parameter and Binding Functions                            ****)
(*******************************************************************)

function cgGetNextBind(program_: PcgProgramIter; iter: PcgBindIter): PcgBindIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetNextBind}

function cgGetBindByName(program_: PcgProgramIter; const parameter_name: PChar): PcgBindIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindByName}

function cgDuplicateBindIter(bind: PcgBindIter): PcgBindIter; cdecl; external CgLibrary;
{$EXTERNALSYM cgDuplicateBindIter}

procedure cgFreeBindIter(iter: PcgBindIter); cdecl; external CgLibrary;
{$EXTERNALSYM cgFreeBindIter}

function cgGetBindParamName(const bind: PcgBindIter): PChar { const }; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindParamName}

function cgGetBindParamType(const bind: PcgBindIter): TcgParamType; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindParamType}

function cgGetBindLocation(const bind: PcgBindIter): TcgBindLocation; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindLocation}

function cgGetBindAddress(const bind: PcgBindIter): Longword; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindAddress}

function cgGetBindValueType(const bind: PcgBindIter; out array_size: Integer): TcgValueType; cdecl; external CgLibrary;
{$EXTERNALSYM cgGetBindValueType}

