{******************************************************************************}
{*                                                                            *}
{*  Copyright (c) 2002, NVIDIA Corporation.                                   *}
{*                                                                            *}
{*  Files:    cgGL.h                                                          *}
{*  Content:  NVIDIA Cg OpenGL interface include files                        *}
{*                                                                            *}
{*  NVIDIA "Cg" ObjectPascal adaptation by Alexey Barkovoy                    *}
{*  E-Mail: clootie@reactor.ru                                                *}
{*                                                                            *}
{*  Modified: 26-Aug-2002                                                     *}
{*                                                                            *}
{*  Latest version can be downloaded from:                                    *}
{*     http://clootie.narod.ru                                                *}
{*     http://delphi-jedi.org                                                 *}
{*                                                                            *}
{******************************************************************************}
{                                                                              }
{ Obtained through: Joint Endeavour of Delphi Innovators (Project JEDI)        }
{                                                                              }
{ The contents of this file are used with permission, subject to the Mozilla   }
{ Public License Version 1.1 (the "License"); you may not use this file except }
{ in compliance with the License. You may obtain a copy of the License at      }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ Alternatively, the contents of this file may be used under the terms of the  }
{ GNU Lesser General Public License (the  "LGPL License"), in which case the   }
{ provisions of the LGPL License are applicable instead of those above.        }
{ If you wish to allow use of your version of this file only under the terms   }
{ of the LGPL License and not to allow others to use your version of this file }
{ under the MPL, indicate your decision by deleting  the provisions above and  }
{ replace  them with the notice and other provisions required by the LGPL      }
{ License.  If you do not delete the provisions above, a recipient may use     }
{ your version of this file under either the MPL or the LGPL License.          }
{                                                                              }
{ For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html }
{                                                                              }
{******************************************************************************}

(*
 *
 * Copyright (c) 2002, NVIDIA Corporation.
 *
 *
 *
 * NVIDIA Corporation("NVIDIA") supplies this software to you in consideration
 * of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this NVIDIA software constitutes
 * acceptance of these terms.  If you do not agree with these terms, please do
 * not use, install, modify or redistribute this NVIDIA software.
 *
 *
 *
 * In consideration of your agreement to abide by the following terms, and
 * subject to these terms, NVIDIA grants you a personal, non-exclusive license,
 * under NVIDIAs copyrights in this original NVIDIA software (the "NVIDIA
 * Software"), to use, reproduce, modify and redistribute the NVIDIA
 * Software, with or without modifications, in source and/or binary forms;
 * provided that if you redistribute the NVIDIA Software, you must retain the
 * copyright notice of NVIDIA, this notice and the following text and
 * disclaimers in all such redistributions of the NVIDIA Software. Neither the
 * name, trademarks, service marks nor logos of NVIDIA Corporation may be used
 * to endorse or promote products derived from the NVIDIA Software without
 * specific prior written permission from NVIDIA.  Except as expressly stated
 * in this notice, no other rights or licenses express or implied, are granted
 * by NVIDIA herein, including but not limited to any patent rights that may be
 * infringed by your derivative works or by other works in which the NVIDIA
 * Software may be incorporated. No hardware is licensed hereunder.
 *
 *
 *
 * THE NVIDIA SOFTWARE IS BEING PROVIDED ON AN "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
 * WITHOUT LIMITATION, WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR ITS USE AND OPERATION
 * EITHER ALONE OR IN COMBINATION WITH OTHER PRODUCTS.
 *
 *
 *
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * EXEMPLARY, CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOST
 * PROFITS; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) OR ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE NVIDIA SOFTWARE,
 * HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING
 * NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF NVIDIA HAS BEEN ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

//////////////////////////////////////////////////////////////////////////////
// HISTORY:
// 26-Aug-02 - Alexey Barkovoy:
//   - Updated to Beta2 of Cg toolkit (compiler version 1.5)
// 16-Jul-02 - Alexey Barkovoy:
//   - Links to new 1.0.1 DLL version
// 01-Jul-02 - Alexey Barkovoy:
//   - Links to single DLL, uses cg.inc file
// 18-Jun-02 - Eric Grange:
//   - TcgGLMatrixFormat enum modified to bunch of constants
//   - Array type in function parameters are removed

unit cgGL;

{$Include JEDI.inc}

interface

(*$HPPEMIT '#include "cgGL.h"' *)

uses
  OpenGL12{$IFNDEF COMPILER6_UP}, Windows{$ENDIF};

{$Include cg.inc}

(*****************************************************************************)
(*** cgGL Type Definitions                                                 ***)
(*****************************************************************************)

type
  TcgGLMatrixFormat = type Cardinal;
  cgGLMatrixFormat = TcgGLMatrixFormat;
  {$NODEFINE cgGLMatrixFormat}
  {$NODEFINE TcgGLMatrixFormat}
  {$HPPEMIT 'typedef cgGLMatrixFormat TcgGLMatrixFormat;'}

const
  cgGLMatrixIdentity    = TcgGLMatrixFormat(0);
  {$EXTERNALSYM cgGLMatrixIdentity}
  cgGLMatrixTranspose   = TcgGLMatrixFormat(1);
  {$EXTERNALSYM cgGLMatrixTranspose}
  cgGLMatrixInverse     = TcgGLMatrixFormat(2);
  {$EXTERNALSYM cgGLMatrixInverse}

type
  TcgGLMatrixType = (
    cgGLModelViewMatrix,
    cgGLProjectionMatrix,
    cgGLTextureMatrix,
    cgGLModelViewProjectionMatrix
  );
  cgGLMatrixType = TcgGLMatrixType;
  {$NODEFINE cgGLMatrixType}
  {$NODEFINE TcgGLMatrixType}
  {$HPPEMIT 'typedef cgGLMatrixType TcgGLMatrixType;'}

(*****************************************************************************)
(*** Program Loading and Binding Functions                                 ***)
(*****************************************************************************)

function cgGLLoadProgram(program_: PcgProgramIter; prog_id: GLuint): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLLoadProgram}
function cgGLBindProgram(program_: PcgProgramIter): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindProgram}

function cgGLEnableProgramType(profile: TcgProfileType): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLEnableProgramType}
function cgGLDisableProgramType(profile: TcgProfileType): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLDisableProgramType}

(*****************************************************************************)
(*** Texture Unit Identifier Functions                                     ***)
(*****************************************************************************)

function cgGLTextureUnit(const param: PcgBindIter): GLenum; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLTextureUnit}
function cgGLActiveTexture(const param: PcgBindIter): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLActiveTexture}
function cgGLClientActiveTexture(const param: PcgBindIter): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLClientActiveTexture}

(*****************************************************************************)
(*** Uniform Parameter Binding Functions                                   ***)
(*****************************************************************************)
{$IFNDEF COMPILER6_UP}
type
  PSingle = ^Single;
  PDouble = ^Double;
{$ENDIF}

function cgGLBindUniform4f(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x, y, z, w: Single): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniform4f}

function cgGLBindUniform4d(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x, y, z, w: Double): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniform4d}

function cgGLBindUniform4fv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PSingle): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniform4fv}

function cgGLBindUniform4dv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PDouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniform4dv}

function cgGLBindUniformMatrixrf(program_: PcgProgramIter;
                                 param: PcgBindIter;
                                 const matrix: PSingle): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniformMatrixrf}

function cgGLBindUniformMatrixrd(program_: PcgProgramIter;
                                 param: PcgBindIter;
                                 const matrix: PDouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniformMatrixrd}

function cgGLBindUniformMatrixcf(program_: PcgProgramIter;
                                 param: PcgBindIter;
                                 const matrix: PSingle): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniformMatrixcf}

function cgGLBindUniformMatrixcd(program_: PcgProgramIter;
                                 param: PcgBindIter;
                                 const matrix: PDouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniformMatrixcd}

function cgGLBindUniformStateMatrix(program_: PcgProgramIter;
                                    param: PcgBindIter;
                                    matrix_type: TcgGLMatrixType;
                                    format: TcgGLMatrixFormat{integer}): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindUniformStateMatrix}

(*****************************************************************************)
(*** Varying Parameter Binding Functions                                   ***)
(*****************************************************************************)
type
  PGLvoid = Pointer;

function cgGLEnableClientState(program_: PcgProgramIter;
                               param: PcgBindIter): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLEnableClientState}

function cgGLDisableClientState(program_: PcgProgramIter;
                                param: PcgBindIter): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLDisableClientState}

function cgGLBindVaryingPointer(program_: PcgProgramIter;
                                param: PcgBindIter;
                                fsize: GLint;
                                type_: GLenum;
                                stride: GLsizei;
                                pointer: PGLvoid): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVaryingPointer}

function cgGLBindVarying1s(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1s}

function cgGLBindVarying1f(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1f}

function cgGLBindVarying1d(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1d}

function cgGLBindVarying2s(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLshort;
                           y: GLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2s}

function cgGLBindVarying2f(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLfloat;
                           y: GLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2f}

function cgGLBindVarying2d(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLdouble;
                           y: GLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2d}

function cgGLBindVarying3s(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLshort;
                           y: GLshort;
                           z: GLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3s}

function cgGLBindVarying3f(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLfloat;
                           y: GLfloat;
                           z: GLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3f}

function cgGLBindVarying3d(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLdouble;
                           y: GLdouble;
                           z: GLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3d}

function cgGLBindVarying4s(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLshort;
                           y: GLshort;
                           z: GLshort;
                           w: GLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4s}

function cgGLBindVarying4f(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLfloat;
                           y: GLfloat;
                           z: GLfloat;
                           w: GLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4f}

function cgGLBindVarying4d(program_: PcgProgramIter;
                           param: PcgBindIter;
                           x: GLdouble;
                           y: GLdouble;
                           z: GLdouble;
                           w: GLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4d}

function cgGLBindVarying4ub(program_: PcgProgramIter;
                            param: PcgBindIter;
                            x: GLubyte;
                            y: GLubyte;
                            z: GLubyte;
                            w: GLubyte): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4ub}

function cgGLBindVarying1sv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1sv}

function cgGLBindVarying1fv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1fv}

function cgGLBindVarying1dv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying1dv}

function cgGLBindVarying2sv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2sv}

function cgGLBindVarying2fv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2fv}

function cgGLBindVarying2dv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying2dv}

function cgGLBindVarying3sv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3sv}

function cgGLBindVarying3fv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3fv}

function cgGLBindVarying3dv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying3dv}

function cgGLBindVarying4sv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLshort): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4sv}

function cgGLBindVarying4fv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLfloat): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4fv}

function cgGLBindVarying4dv(program_: PcgProgramIter;
                            param: PcgBindIter;
                            const v: PGLdouble): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4dv}

function cgGLBindVarying4ubv(program_: PcgProgramIter;
                             param: PcgBindIter;
                             const v: PGLubyte): TcgError; cdecl; external CgLibrary;
{$EXTERNALSYM cgGLBindVarying4ubv}

implementation

end.
